/*
matrix.c
Author: Hayden Walles
Date: 12 September 2007
Implementation of the matrix datatype.
*/

/*
    Copyright (C) 2007 Hayden Walles

    This file is part of Seamstress.

    Seamstress is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Seamstress is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include "seams.h"


/*newMatrix tries to create a new matrix of the specified size and format.  Returns pointer to the new matrix if successful, or NULL if there is not enough memory, or an invalid format was given.  If error is non-NULL, returns an error value in *error.*/
MATRIX *newMatrix(int rows,int cols, int format,int *error){
  MATRIX *matrix;
 
  /*Grab some memory for the MATRIX ADT*/
  matrix=malloc(sizeof(MATRIX));
  if(matrix==NULL){
    if(error!=NULL)
      *error=MATRIX_E_MEMORY;
    return NULL;
  }

  /*Now obtain the buffer to store actual data.  This varies depending on format*/
  switch(format){
    case MT_BYTE:
    matrix->buffer.bytes=malloc(cols*rows*sizeof(unsigned char));
    if(matrix->buffer.bytes==NULL){
      free(matrix);
    if(error!=NULL)
      *error=MATRIX_E_MEMORY;
      return NULL;
    }
    break;  
  case MT_INTEGER:
    matrix->buffer.integers=malloc(cols*rows*sizeof(int));
    if(matrix->buffer.integers==NULL){
      free(matrix);
    if(error!=NULL)
      *error=MATRIX_E_MEMORY;
      return NULL;
    }
    break;
  case MT_REAL:
    matrix->buffer.reals=malloc(cols*rows*sizeof(double));
    if(matrix->buffer.reals==NULL){
      free(matrix);
    if(error!=NULL)
      *error=MATRIX_E_MEMORY;
      return NULL;
    }
    break;
#if USE_COMPLEX
  case MT_COMPLEX:
    matrix->buffer.complexes=malloc(cols*rows*sizeof(COMPLEX));
    if(matrix->buffer.complexes==NULL){
      free(matrix);
    if(error!=NULL)
      *error=MATRIX_E_MEMORY;
      return NULL;
    }
    break;
#endif
  default:
    free(matrix);
    if(error!=NULL)
      *error=MATRIX_E_WRONGTYPE;
    return NULL;
  }

  /*Finish setting the ADT values*/
  matrix->rows=rows;
  matrix->cols=cols;
  matrix->type=format;
  

  return matrix;


}



/*deleteMatrix frees the memory associated with the specified matrix.*/
void deleteMatrix(MATRIX *matrix){
  switch(matrix->type){
  case MT_INTEGER:
    free(matrix->buffer.integers);
    break;
  case MT_REAL:
    free(matrix->buffer.reals);
    break;
#ifdef USE_COMPLEX
  case MT_COMPLEX:
    free(matrix->buffer.complexes);
    break;
#endif
  case MT_BYTE:
    free(matrix->buffer.bytes);
    break;
  }
  free(matrix);
}


