/*
picture.c
Author: Hayden Walles
Date: 18 September 2007
Shared functions for setting the luminance matrix of unpickers and unweavers.
*/

/*
    Copyright (C) 2007 Hayden Walles

    This file is part of Seamstress.

    Seamstress is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Seamstress is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "seams.h"
#define LUM(r,g,b) ((299*(r)+587*(g)+144*(b))/1030)


void setPictureRowRGB32(MATRIX *lum,int hborder,int vborder, int transposed,int row, unsigned char *pixels){

  int i,j;
  int pix;
  int span;

  if(transposed==0){
    /*Non transposed luminance.*/
    span=matrixCols(lum)-2*hborder;
    i=row+vborder;
    pix=LUM(pixels[0],pixels[1],pixels[2]);
    for(j=0;j<=hborder;j++)
      setInteger(lum,i,j,pix);
    for(j=1;j<span;j++)
      setInteger(lum,i,j+hborder,LUM(pixels[j*4],pixels[j*4+1],pixels[j*4+2]));
    pix=getInteger(lum,i,j-1+hborder);
    for(j=0;j<hborder;j++)
      setInteger(lum,i,span+hborder+j,pix);
    
    if(row==0){
      for(i=vborder-1;i>=0;i--){
	for(j=0;j<matrixCols(lum);j++)
	  setInteger(lum,i,j,getInteger(lum,i+1,j));
      }
    }
    else if(row==matrixRows(lum)-2*vborder-1){
      for(i=matrixRows(lum)-vborder;i<matrixRows(lum);i++){
	for(j=0;j<matrixCols(lum);j++)
	  setInteger(lum,i,j,getInteger(lum,i-1,j));
      }
    }

  }
  else {
    /*Transposed luminance.*/
    span=matrixRows(lum)-2*vborder;
    j=row+hborder;
    pix=LUM(pixels[0],pixels[1],pixels[2]);
    for(i=0;i<=vborder;i++)
      setInteger(lum,i,j,pix);
    for(i=1;i<span;i++)
      setInteger(lum,i+vborder,j,LUM(pixels[i*4],pixels[i*4+1],pixels[i*4+2]));
    pix=getInteger(lum,i-1+vborder,j);
    for(i=0;i<vborder;i++)
      setInteger(lum,span+vborder+i,j,pix);
    
    if(row==0){
      for(j=hborder-1;j>=0;j--){
	for(i=0;i<matrixRows(lum);i++)
	  setInteger(lum,i,j,getInteger(lum,i,j+1));
      }
    }
    else if(row==matrixCols(lum)-2*hborder-1){
      for(j=matrixCols(lum)-hborder;j<matrixCols(lum);j++){
	for(i=0;i<matrixRows(lum);i++)
	  setInteger(lum,i,j,getInteger(lum,i,j-1));
      }
    }
  }

  return;

}


void setPictureRowRGB24(MATRIX *lum,int hborder,int vborder, int transposed,int row, unsigned char *pixels){

  int i,j;
  int pix;
  int span;

  if(transposed==0){
    /*Non transposed luminance.*/
    span=matrixCols(lum)-2*hborder;
    i=row+vborder;
    pix=LUM(pixels[0],pixels[1],pixels[2]);
    for(j=0;j<=hborder;j++)
      setInteger(lum,i,j,pix);
    for(j=1;j<span;j++)
      setInteger(lum,i,j+hborder,LUM(pixels[j*3],pixels[j*3+1],pixels[j*3+2]));
    pix=getInteger(lum,i,j-1+hborder);
    for(j=0;j<hborder;j++)
      setInteger(lum,i,span+hborder+j,pix);
    
    if(row==0){
      for(i=vborder-1;i>=0;i--){
	for(j=0;j<matrixCols(lum);j++)
	  setInteger(lum,i,j,getInteger(lum,i+1,j));
      }
    }
    else if(row==matrixRows(lum)-2*vborder-1){
      for(i=matrixRows(lum)-vborder;i<matrixRows(lum);i++){
	for(j=0;j<matrixCols(lum);j++)
	  setInteger(lum,i,j,getInteger(lum,i-1,j));
      }
    }

  }
  else {
    /*Transposed luminance.*/
    span=matrixRows(lum)-2*vborder;
    j=row+hborder;
    pix=LUM(pixels[0],pixels[1],pixels[2]);
    for(i=0;i<=vborder;i++)
      setInteger(lum,i,j,pix);
    for(i=1;i<span;i++)
      setInteger(lum,i+vborder,j,LUM(pixels[i*3],pixels[i*3+1],pixels[i*3+2]));
    pix=getInteger(lum,i-1+vborder,j);
    for(i=0;i<vborder;i++)
      setInteger(lum,span+vborder+i,j,pix);
    
    if(row==0){
      for(j=hborder-1;j>=0;j--){
	for(i=0;i<matrixRows(lum);i++)
	  setInteger(lum,i,j,getInteger(lum,i,j+1));
      }
    }
    else if(row==matrixCols(lum)-2*hborder-1){
      for(j=matrixCols(lum)-hborder;j<matrixCols(lum);j++){
	for(i=0;i<matrixRows(lum);i++)
	  setInteger(lum,i,j,getInteger(lum,i,j-1));
      }
    }
  }

  return;

}


